import nl4py
import logging
import pytest
import sys
import time

logger = logging.getLogger('suite_agents')

#white-box test
def test_initial_wage_sicks(setup_model):
    log = logging.getLogger("test_initial_wage_sicks")
    log.debug("Lower initial-wage means more sick employees.")
    setup_model.workspace.command("set initial-wage 50")
    time.sleep(.500)
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(1)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(1)
    value_less_wage = setup_model.workspace.report("count turtles with [ is-sick? = true]")
    time.sleep(.500)
    setup_model.workspace.command("set initial-wage 200")
    time.sleep(.500)
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(1)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(1)
    value_more_wage = setup_model.workspace.report("count turtles with [ is-sick? = true]")
    assert value_less_wage > value_more_wage

def test_sick_slider_compare(setup_model):
    logger.debug("Sick-slider set for longer period means more sick agents in general")
    setup_model.workspace.command("set sick-slider 100")
    time.sleep(.500)
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(1)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1)
    sick_slider_more = setup_model.workspace.report("count turtles with [ is-sick? ]")
    time.sleep(.500)
    setup_model.workspace.command("set sick-slider 7")
    time.sleep(.500)
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(1)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1)
    sick_slider_less = setup_model.workspace.report("count turtles with [ is-sick? ]")
    time.sleep(.500)
    assert sick_slider_more > sick_slider_less

def test_retirement_off(setup_model):
    logger.debug("With retirement switched off, employees cannot be fired")
    setup_model.workspace.command("set retirement false")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(.500)
    fired_employees = setup_model.workspace.report("sum count-fired-sick + sum count-fired-performance")
    assert fired_employees == 0

def test_fixed_performance_on(setup_model):
    logger.debug("With fixed-performance switched on, average performance of employees is not changing")
    setup_model.workspace.command("set fixed-performance true")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(2)
    performance = setup_model.workspace.report("sum [own-performance] of turtles / num-nodes")
    time.sleep(.500)
    assert performance == 100

def test_fixed_performance_off(setup_model):
    logger.debug("With fixed-performance switched off, average performance of employees is changing")
    setup_model.workspace.command("set fixed-performance false")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(2)
    performance = setup_model.workspace.report("sum [own-performance] of turtles / num-nodes")
    time.sleep(.500)
    assert performance != 100

def test_effort_change_on(setup_model):
    logger.debug("With effort-change switched on, average effort of employees is changing")
    setup_model.workspace.command("set effort-change true")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(2)
    effort = setup_model.workspace.report("sum [effort] of turtles / num-nodes")
    time.sleep(.500)
    assert effort != 75

def test_effort_change_off(setup_model):
    logger.debug("With effort-change switched on, average effort of employees is not changing")
    setup_model.workspace.command("set effort-change false")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(2)
    effort = setup_model.workspace.report("sum [effort] of turtles / num-nodes")
    assert effort == 75

def test_minimal_productivity_params(setup_model):
    logger.debug("Mean-value-productivity and std-deviation-productivity on minimum means negative total company value")
    setup_model.workspace.command("set mean-value-productivity 2.1")
    setup_model.workspace.command("set std-deviation-productivity 0.05")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(.500)
    company_value = setup_model.workspace.report("total-company-value")
    assert company_value < 0

def test_maximal_productivity_params(setup_model):
    logger.debug("Mean-value-productivity and std-deviation-productivity on maximum means positive total company value")
    setup_model.workspace.command("set mean-value-productivity 5")
    setup_model.workspace.command("set std-deviation-productivity 1.5")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(.500)
    company_value = setup_model.workspace.report("total-company-value")
    assert company_value > 0

def test_performance_limits(setup_model):
    logger.debug("Lower performance-lower-limit and performance-upper-limit means lower own-performance of turtles")
    setup_model.workspace.command("set performance-lower-limit 25")
    setup_model.workspace.command("set performance-upper-limit 110")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(2)
    performance_lower_limits = setup_model.workspace.report("mean [own-performance] of turtles")
    time.sleep(.500)
    setup_model.workspace.command("set performance-lower-limit 80")
    setup_model.workspace.command("set performance-upper-limit 300")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(2)
    performance_higher_limits = setup_model.workspace.report("mean [own-performance] of turtles")
    time.sleep(.500)
    assert performance_lower_limits < performance_higher_limits

