import nl4py
import logging
import pytest
import sys
import time

logger = logging.getLogger('suite_basic_logic')

def test_cooperation_pays_off(setup_model):
    logger.debug("Agents with higher cooperation coeficient bring more value to the company.")
    setup_model.workspace.command("setup-barabasi-albert")
    setup_model.workspace.command("set initial-cooperation 0")
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(1)
    value_non_coop = setup_model.workspace.report("total-company-value")
    time.sleep(.500)
    setup_model.workspace.command("setup-barabasi-albert")
    setup_model.workspace.command("set initial-cooperation 1")
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(1)
    value_coop = setup_model.workspace.report("total-company-value")
    time.sleep(.500)
    assert value_non_coop < value_coop

def test_initial_wage(setup_model):
    logger.debug("Initial Total-company-value is made of initial-wage.")
    setup_model.workspace.command("set initial-wage 150")
    setup_model.workspace.command("set num-nodes 100")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    value = setup_model.workspace.report("total-company-value")
    time.sleep(.500)
    assert value == 15000

def test_cooperation_part_max(setup_model):
    log = logging.getLogger("test_cooperation_part_max")
    log.debug("Maximal cooperation-part changes self-evaluation of turtles")
    setup_model.workspace.command("set cooperation-part 0.99")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(1)
    value = setup_model.workspace.report("sum [self-evaluation] of turtles")
    assert value != 0

def test_penalisation_for_fluctuation(setup_model):
    logger.debug("Company with higher penalisation-for-fluctuation will have lower total value")
    setup_model.workspace.command("set penalisation-for-fluctuation 0")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(2)
    value_with_no_penalization = setup_model.workspace.report("total-company-value")
    setup_model.workspace.command("set penalisation-for-fluctuation 12")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(2)
    value_with_max_penalization = setup_model.workspace.report("total-company-value")
    assert value_with_no_penalization > value_with_max_penalization

@pytest.mark.xfail
def test_agent_cooperates_is_defected(setup_model):
    logger.debug("If agent chooses to cooperate and is defected by his partner, he losts value")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 1 [go]")
    time.sleep(.500)
    value_balancing_PD = setup_model.workspace.report("ask one-of turtles with [not defect-now? and partner-defected?] [output generated-value-balancing-PD]")
    value_balancing_PD = int(value_balancing_PD)
    assert value_balancing_PD < 0

@pytest.mark.xfail
def test_agent_defects_is_cooperated_with(setup_model):
    logger.debug("If agent chooses to defect and his partner cooperates, he gains value")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 1 [go]")
    time.sleep(.500)
    value_balancing_PD = setup_model.workspace.report("ask one-of turtles with [defect-now? and not partner-defected?] [output generated-value-balancing-PD]")
    value_balancing_PD = int(value_balancing_PD)
    assert value_balancing_PD > 0
