import nl4py
import logging
import pytest
import sys
import time

logger = logging.getLogger('suite_management')

def test_patience_fired_sick(setup_model):
    logger.debug("Lower boss patience means more fired employees for sickness")
    setup_model.workspace.command("set patience 6")
    setup_model.workspace.command("setup-barabasi-albert")
    setup_model.workspace.command("repeat 1080 [go]")  #year and a half
    time.sleep(4)
    value_less_patience = setup_model.workspace.report("sum-fired-sick")
    time.sleep(2)
    setup_model.workspace.command("set patience 24")
    setup_model.workspace.command("setup-barabasi-albert")
    setup_model.workspace.command("repeat 1080 [go]")
    time.sleep(4)
    value_more_patience = setup_model.workspace.report("sum-fired-sick")
    time.sleep(2)
    assert value_less_patience > value_more_patience

def test_patience_fired_performance(setup_model):
    logger.debug("Lower boss patience means more fired employees for bad performance.")
    setup_model.workspace.command("set patience 6")
    setup_model.workspace.command("setup-barabasi-albert")
    setup_model.workspace.command("repeat 1080 [go]")
    time.sleep(4)
    value_less_patience = setup_model.workspace.report("sum-fired-performance")
    time.sleep(2)
    setup_model.workspace.command("set patience 24")
    setup_model.workspace.command("setup-barabasi-albert")
    setup_model.workspace.command("repeat 1080 [go]")
    time.sleep(4)
    value_more_patience = setup_model.workspace.report("sum-fired-performance")
    time.sleep(2)
    assert value_less_patience > value_more_patience

def test_sickness_probability_fired(setup_model):
    logger.debug("Higher sickness probability means more fired employees for sickness")
    setup_model.workspace.command("set sickness-probability 0.1")
    setup_model.workspace.command("setup-barabasi-albert")
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1.5)
    fired_sick = int(setup_model.workspace.report("sum-fired-sick"))
    fired_performance = int(setup_model.workspace.report("sum-fired-performance"))
    value_less_probable = fired_sick + fired_performance
    time.sleep(1)
    setup_model.workspace.command("set sickness-probability 1")
    setup_model.workspace.command("setup-barabasi-albert")
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1.5)
    fired_sick = int(setup_model.workspace.report("sum-fired-sick"))
    fired_performance = int(setup_model.workspace.report("sum-fired-performance"))
    value_more_probable = fired_sick + fired_performance
    time.sleep(1)
    assert value_less_probable < value_more_probable

def test_budget_changes_zero(setup_model):
    logger.debug("With budget variables set to zero, there will be no change in total budget")
    setup_model.workspace.command("set budget-change 0")
    setup_model.workspace.command("set fixed-budget-change 0")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    initial_budget = setup_model.workspace.report("initial-budget")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(.500)
    budget = setup_model.workspace.report("budget")
    time.sleep(.500)
    assert budget == initial_budget

def test_budget_change(setup_model):
    logger.debug("Higher budget-change means bigger leaps (negative or positive) on overall budget ")
    setup_model.workspace.command("set budget-change 0.1")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat boss-reaction-time + 1 [go]") # default boss-reaction-time is 30 ticks
    time.sleep(1)
    higher_budget_change_budget = setup_model.workspace.report("budget")
    time.sleep(.500)
    setup_model.workspace.command("set budget-change 0.01")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat boss-reaction-time + 1 [go]")
    time.sleep(1)
    lower_budget_change_budget = setup_model.workspace.report("budget")
    assert abs(higher_budget_change_budget) >  abs(lower_budget_change_budget)

def test_fixed_budget(setup_model):
    logger.debug("Budget gets increased fixed-budget")
    setup_model.workspace.command("set fixed-budget-change 150")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat boss-reaction-time + 1 [go]") # default boss-reaction-time is 30 ticks
    time.sleep(.500)
    budget = setup_model.workspace.report("budget")
    time.sleep(.500)
    assert budget == 10000

def test_minimal_budget_equals_maximal(setup_model):
    logger.debug("Minimal-budget cannot be equal to maximal-budget in order to change overall budget")
    setup_model.workspace.command("set minimal-budget 1")
    setup_model.workspace.command("set maximal-budget 1")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(.500)
    budget = setup_model.workspace.report("budget")
    assert budget == 10000

def test_less_ticks_than_boss_reaction_time(setup_model):
    logger.debug("There will be no change in budget before number of ticks exceeds boss-reaction-time")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat boss-reaction-time - 1 [go]")
    time.sleep(.500)
    budget = setup_model.workspace.report("budget")
    assert budget == 10000

def test_more_ticks_than_boss_reaction_time(setup_model):
    logger.debug("Generated value is changed after number of ticks exceeds boss-reaction-time")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat boss-reaction-time + 1 [go]")
    time.sleep(1)
    budget = setup_model.workspace.report("sum [sum-generated-value] of turtles")
    time.sleep(.500)
    assert budget != 10000

def test_boss_reaction_intensity_on_high_productivity(setup_model):
    logger.debug("The higher intensity of boss reaction, the more positive feedback on good productivity of employee")
    setup_model.workspace.command("set boss-reaction-intensity 0.9")
    setup_model.workspace.command("set mean-value-productivity 5")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(2)
    self_evaluation_intensive_reaction = setup_model.workspace.report("sum [own-performance] of turtles")
    setup_model.workspace.command("set boss-reaction-intensity 0.1")
    setup_model.workspace.command("set mean-value-productivity 5")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(2)
    self_evaluation_not_intensive_reaction = setup_model.workspace.report("sum [own-performance] of turtles")
    assert self_evaluation_intensive_reaction > self_evaluation_not_intensive_reaction

def test_boss_reaction_intensity_on_low_productivity(setup_model):
    logger.debug("The higher intensity of boss reaction, the more negative feedback on bad productivity of employee")
    setup_model.workspace.command("set boss-reaction-intensity 0.9")
    setup_model.workspace.command("set mean-value-productivity 2.1")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(1)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(1)
    self_evaluation_intensive_reaction = setup_model.workspace.report("sum [own-performance] of turtles")
    setup_model.workspace.command("set boss-reaction-intensity 0.1")
    setup_model.workspace.command("set mean-value-productivity 2.1")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(1)
    setup_model.workspace.command("repeat 360 [go]")
    time.sleep(1)
    self_evaluation_not_intensive_reaction = setup_model.workspace.report("sum [own-performance] of turtles")
    assert self_evaluation_intensive_reaction < self_evaluation_not_intensive_reaction

def test_boss_insight_cooperation_probability_zero(setup_model):
    logger.debug("Zero boss-insight-cooperation-probability means, there will be no stolen generated value")
    setup_model.workspace.command("set boss-insight-cooperation-probability 0")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(.500)
    value_balancing_PD = setup_model.workspace.report("sum [generated-value-balancing-PD] of turtles")
    assert value_balancing_PD == 0

def test_boss_insight_cooperation_part_zero(setup_model):
    logger.debug("Zero boss-insight-cooperation-part means, there will be no stolen generated value")
    setup_model.workspace.command("set boss-insight-cooperation-part 0")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(.500)
    value_balancing_PD = setup_model.workspace.report("sum [generated-value-balancing-PD] of turtles")
    assert value_balancing_PD == 0

def test_boss_insight_cooperation_probability_max(setup_model):
    logger.debug("Maximal boss-insight-cooperation-probability means, there will be some stolen generated value")
    setup_model.workspace.command("set boss-insight-cooperation-probability 1")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1)
    value_balancing_PD = setup_model.workspace.report("sum [generated-value-balancing-PD] of turtles")
    time.sleep(.500)
    assert value_balancing_PD != 0

def test_boss_insight_cooperation_part_max(setup_model):
    logger.debug("Max boss-insight-cooperation-part means, there will be some stolen generated value")
    setup_model.workspace.command("set boss-insight-cooperation-part 1")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1)
    value_balancing_PD = setup_model.workspace.report("sum [generated-value-balancing-PD] of turtles")
    time.sleep(.500)
    assert value_balancing_PD != 0

def test_boss_insight_performance_zero(setup_model):
    logger.debug("Zero boss-insight-cooperation-part means, there will be no stolen generated value")
    setup_model.workspace.command("set boss-insight-performance 0")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1)
    evaluation = setup_model.workspace.report("sum [self-evaluation] of turtles")
    time.sleep(.500)
    assert evaluation <= 0

def test_boss_insight_performance_max(setup_model):
    logger.debug("Maximal boss-insight-cooperation-probability means, the self evaluation of turtles will increase")
    setup_model.workspace.command("set boss-insight-performance 1")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1)
    evaluation = setup_model.workspace.report("sum [self-evaluation] of turtles")
    time.sleep(.500)
    assert evaluation != 0
