import nl4py
import logging
import pytest
import sys
import time

logger = logging.getLogger('suite_stress')

def test_min_vs_max_stress_params(setup_model):
    logger.debug("Higher mean-value-stress-limit and std-deviation-stress-limit means employees can handle more stress.")
    setup_model.workspace.command("set mean-value-stress-limit 500")
    setup_model.workspace.command("set std-deviation-stress-limit 10")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1)
    stress_with_max_params = setup_model.workspace.report("sum [stress-actual] of turtles")
    time.sleep(.500)
    setup_model.workspace.command("set mean-value-stress-limit 100")
    setup_model.workspace.command("set std-deviation-stress-limit 1")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1)
    stress_with_min_params = setup_model.workspace.report("sum [stress-actual] of turtles")
    time.sleep(.500)
    assert stress_with_max_params > stress_with_min_params

def test_stress_regen(setup_model):
    logger.debug("Employees with higher stress-regen don't get sick as much as employees with lower regeneration")
    setup_model.workspace.command("set stress-regen 0")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 30 [go]")
    time.sleep(1)
    sickos_less_regen = setup_model.workspace.report("count turtles with [is-sick?]")
    time.sleep(.500)
    setup_model.workspace.command("set stress-regen 2")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 30 [go]")
    time.sleep(1)
    sickos_more_regen = setup_model.workspace.report("count turtles with [is-sick?]")
    time.sleep(.500)
    assert sickos_less_regen > sickos_more_regen

def test_stress_modification_on_PD(setup_model):
    logger.debug("Employees who defected and have higher stress-modification-on-PD experience more stress than employees with lower stress-modification-on-PD who defected")
    setup_model.workspace.command("set stress-modification-on-PD 0")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 30 [go]")
    time.sleep(.500)
    stress_less_modification = setup_model.workspace.report("sum [stress-actual] of turtles")
    setup_model.workspace.command("set stress-modification-on-PD 0.5")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 30 [go]")
    time.sleep(.500)
    stress_more_modification = setup_model.workspace.report("sum [stress-actual] of turtles")
    assert stress_less_modification < stress_more_modification

def test_evaluation_stress_change(setup_model):
    logger.debug("Hugher evaluation-stress-change means more stress for employees")
    setup_model.workspace.command("set evaluation-stress-change 0.05")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1)
    stress_less_change = setup_model.workspace.report("sum [stress-actual] of turtles")
    time.sleep(.500)
    setup_model.workspace.command("set evaluation-stress-change 1.9")
    setup_model.workspace.command("setup-barabasi-albert")
    time.sleep(.500)
    setup_model.workspace.command("repeat 180 [go]")
    time.sleep(1)
    stress_more_change = setup_model.workspace.report("sum [stress-actual] of turtles")
    time.sleep(.500)
    assert stress_less_change < stress_more_change
